function [c,condMean,resid,rotResid,costcall,costflag]=cost(b,Z,X,z0,ZbC,costcall,costflag)
%
% - ZbC is a basis in the tangent space of Kendall's planar shape space with
%   nL landmarks;
% - Z must be the output from make2DpreshapeC(); 
%
%vc=B*x
[nL,nSubj]=size(Z);
dM=2*nL-4; %dM is the dimension (over R) of the planar shape space.
dimC=nL-2;

[nSubj1,cov_num]=size(X);
if nSubj ~= nSubj1 
    error('covariates X and responses Y (i.e. Z) are not provided for the same number of subjects');
end
if length(z0) ~= nL || abs(mean(z0))>0.0000000001 || abs(norm(z0)-1)>0.0000000001
    error('z0 and Z are not in the same manifold');
end

% adjust the cost value
costfactor=10000;

nb=length(b);
if mod(nb,dM) ~= 0
    error('length of b is must be a multiple of dM');
end

condMean=zeros(nL,nSubj); 
resid=condMean;
rotResid=condMean;


bC=reshape(b,2,[]).'*[1;1i];
Baug=reshape(bC,dimC,[]);
bq=Baug(:,1);
if norm(bq) >= pi/2 
    c=999999999;
    costflag=99;
else
    csi_bq=ZbC*bq;
    zq=exp2DshapeC(z0,csi_bq);
    if length(bC)==dimC
        mu=zq;
        c=0;
        s=mu'*z0;
        as=s/abs(s);
        for subj=1:nSubj
            condMean(:,subj)=mu;
            resid(:,subj)=log2DshapeC(mu,Z(:,subj));
            rotResid(:,subj)=spUnitaryGeodesic2DshapeC(mu,z0,resid(:,subj));
            rotResid(:,subj)=as*rotResid(:,subj);
            dist=acos(abs(mu'*Z(:,subj)));
            c=c+dist*dist;
        end
        c=c/nSubj*costfactor;
        costflag=0;
    else
        B=Baug;
        B(:,1)=[];
        [rB,cB]=size(B);
        if cB ~= cov_num
            error('matrix B, of covariate parameters, and matrix X must have the same number of columns');
        end
        BX=B*(X.');
        if sum(sum(BX.*BX)>pi*pi/4)>=1;
            c=999999999;
            costflag=98;
        else
            [ZbC_zq,U]=spUnitaryGeodesic2DshapeC(z0,zq,ZbC);
            csi_BX=ZbC_zq*BX; % csi_BX is a dimC x nSubj matrix
                              % each column is a tangent vector at z
            c=0;
            for subj=1:nSubj
                mux=exp2DshapeC(zq,csi_BX(:,subj));
                condMean(:,subj)=mux;
                resid(:,subj)=log2DshapeC(mux,Z(:,subj));
                rotResid(:,subj)=spUnitaryGeodesic2DshapeC(mux,z0,resid(:,subj));
                s=mux'*z0;
                as=s/abs(s);
                rotResid(:,subj)=as*rotResid(:,subj);
                dist=acos(abs(mux'*Z(:,subj)));    
                c=c+dist*dist;
            end
            c=c/nSubj*costfactor;
            costflag=0;
        end
    end
    
end
costcall=costcall+1;
end



