load('ADNI_RMRSS\data\ADinfoM.mat');
load('ADNI_RMRSS\data\ADinfoF.mat');
load('ADNI_RMRSS\data\NCinfoF.mat');
load('ADNI_RMRSS\data\NCinfoM.mat');
load('ADNI_RMRSS\data\ADLdataM.mat');
load('ADNI_RMRSS\data\ADLdataF.mat');
load('ADNI_RMRSS\data\NCLdataM.mat');
load('ADNI_RMRSS\data\NCLdataF.mat');

model=8; % We set up several models; model 8 was the 
         % one used in for the analysis in the paper. 

infoALL=[ADinfoF;ADinfoM;NCinfoF;NCinfoM];
size(infoALL);
LdataALL=cat(3,ADLdataF,ADLdataM,NCLdataF,NCLdataM);
size(LdataALL);

xData=infoALL(:,[2 9 10]); %%% gender, age, diagnosis
[nSubj1,nCov]=size(xData);
xData_orig=xData;
%%% X is a mofified version of XX so that F=0, M=1, NC=0, AD=1;
X=xData;
xData(:,1)=2-xData(:,1);
xData(:,3)=(xData(:,3)-1)/2;
XX=xData;
XXmeans=mean(XX);
XXstds=std(XX);
XXst=(XX-repmat(XXmeans,nSubj1,1))./repmat(XXstds,nSubj1,1);


Ldata=LdataALL;
Ldata_orig=Ldata;
[nL,nCoord,nSubj]=size(Ldata);
if nSubj1 ~= nSubj
    error('covariates and responses must be provided for the same number of subjects');
end

%%% Subsample landmarks
jump=1;
yLdata=Ldata(1:jump:nL,:,:);
% Redefine nL and nSubj
[nL,nCoord,nSubj]=size(yLdata)

%%% Reshuffle the points within landmarks
fp=12;
if fp>1
    yLdata=yLdata([fp:nL 1:(fp-1)],:,:);
end

zLdata=zeros(nL,nSubj);
for subj=1:nSubj
    zLdata(:,subj)=make2DpreshapeC(yLdata(:,:,subj));
end
Z=zLdata;
nL

varsZ=var(real(Z),1,2)+var(imag(Z),1,2);
aux1=[Z(nL,:);Z(1:(nL-1),:)]-Z;
aux2=[Z(2:nL,:);Z(1,:)]-Z;
normAux1=abs(aux1);
normAux2=abs(aux2);
curvZmat=real(aux1.*conj(aux2))./(normAux1.*normAux2); % curZmat(j,k) is 
        % the cosine of the angle between the two adjacent sides in 
        % the CC contour of the k-th subject that meet at the j-th grid
        % point.

meanCurvZ=mean(curvZmat,2);
allInd=(1:nL)';
indV80C75=find(mod(allInd,2)+((1-mod(allInd,2)).*((varsZ>=prctile(varsZ,80))+(meanCurvZ>=prctile(meanCurvZ,75)))>0));

yLdata=yLdata(indV80C75,:,:);
nL=length(indV80C75);
zLdata=zeros(nL,nSubj);
for subj=1:nSubj
    zLdata(:,subj)=make2DpreshapeC(yLdata(:,:,subj));
end
Z=zLdata;
nL

dimC=nL-2;
dM=2*dimC;

% Initial base point z0 in the Kendall's planar space;
z0=zeros(nL,1);
z0(1:2,1)=[1;-1]/sqrt(2);
% ZbC basis over C in the tangent space to Kendall's planar shape at z0;
a1=2:(dimC+1);
a2=sqrt(1./(a1.*(a1+1)));
ZbC=triu(ones(nL,dimC),-1)*diag(a2)-[zeros(2,dimC);diag(a1.*a2)];
%%%%%%%%%%%%%%%% END OF DATA PREPARATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Model Selection:
if model==1 %% gender only
    X0=XX(:,1); 
    X=XXst(:,1); 
    cov_type=[1];
elseif model==2 %% age only
    X0=XX(:,2);
    X=XXst(:,2);
    cov_type=[0];
elseif model==3 %% diagnosis only
    X0=XX(:,3);
    X=XXst(:,3);
    cov_type=[1];
elseif model==4 %% gender and age
    X0=XX(:,1:2);
    X=XXst(:,1:2);
    cov_type=[1 0];
elseif model==5 %% age and diagnosis
    X0=XX(:,2:3);
    X=XXst(:,2:3);
    cov_type=[0 1];
elseif model==6 %%age, diagnosis, and their interaction;
    X0=XX(:,2:3);
    X=[XXst(:,2:3) XXst(:,2).*XXst(:,3)];
    cov_type=[0 1];
elseif model==7 %%gender, age, diagnosis, no interactions;
    X0=XX;
    X=XXst;
    cov_type=[1 0 1];
elseif model==8 %%gender, age, diagnosis, and a*d interaction;
    X0=XX;
    X=[XXst XXst(:,2).*XXst(:,3)]; 
    cov_type=[1 0 1];
end

Y=yLdata;
Z=zLdata;

% redefine 
nSubj=length(Z(1,:));
cov_num=length(X(1,:));
dim=dM*(1+cov_num);
% 'number of parameters is dim'

costfactor=10000;

% Log_{z_0}-transformed responses vZ to T_{z_0}M and their mean 
vZ=log2DshapeCvec(z0,Z);
% vZ=zeros(size(Z));
% len=length(vZ(1,:));
% for subj=1:len
%     vZ(:,subj)=log2DshapeC(z0,Z(:,subj));
% end
vZ_mean=mean(vZ,2);


% Sum of squares \sum_{i=1}^nSubj \|vZ(:,i)-vZ_mean\|^2
z_vZ_mean=exp2DshapeC(z0,vZ_mean);
dists=acos(abs(z_vZ_mean'*Z));
costI_exp_vZ_mean=dists*(dists.')/nSubj*costfactor;

vvZ=log2DshapeCvec(z_vZ_mean,Z);
costI_exp_vZ_mean_2=norm(vvZ,'fro')^2/nSubj*costfactor;

% coordinates of z_vZ_mean in the natural chart centerec at initial z0
coordC_vZ_mean=ZbC'*vZ_mean;
coordR_vZ_meanR=reshape([real(coordC_vZ_mean).';imag(coordC_vZ_mean).'],2,nL-2).';
beta_vZ_mean=coordR_vZ_meanR.';
beta_vZ_mean=beta_vZ_mean(:);
beta_vZ_mean=beta_vZ_mean.';

% redefine z0 and ZbC
z0_orig=z0;
ZbC_orig=ZbC;
[ZbC_z_vZ_mean,U]=spUnitaryGeodesic2DshapeC(z0,z_vZ_mean,ZbC);
z0=z_vZ_mean;
ZbC=ZbC_z_vZ_mean;

coordC_vZ_mean=ZbC'*log2DshapeC(z0,z_vZ_mean);
coordR_vZ_meanR=reshape([real(coordC_vZ_mean).';imag(coordC_vZ_mean).'],2,nL-2).';
beta_vZ_mean=coordR_vZ_meanR.';
beta_vZ_mean=beta_vZ_mean(:);
beta_vZ_mean=beta_vZ_mean.';

costcall=0;
costflag=0;
[c0,~,~,~,costcall,costflag]=cost(beta_vZ_mean,Z,X,z0,ZbC,costcall,costflag);

