function [W,U]=spUnitaryGeodesic2DshapeC(z1,z2,V)
%
% Calculates the special unitary map that moves z1 over z2ztar and 
% acts on span_C(z1,z2)=span_C(z1,z2star);
% (z2star is in span_C(z1,z2)).
%


% if length(z1) ~= length(z2) || length(z1) ~= length(v)
%     error('vectors z1, z2, v, must have the same length');
% end
% 
% s=z2'*z1;   % inner porduct <z1,z2>_C;
% z2=(s/abs(s))*z2;   % z2 is now z2star;
% s=abs(s);   % inner porduct <z1,z2star>_C, which is positive real number,
%             % so s=<z1,z2star>_C=<z2star,z1>_C.
% b=sqrt(1-s*s);
% z2tilda=(z2-s*z1)/b;  % z2tilda is a unit vector in span_C(z1,z2) such 
%                       % that it is othogonal (over C) to z1, 
%                       % i.e <z1,z2tilda>_C=0. 
% U=[s -b; b s];
% vc=[z1'*v; z2tilda'*v];  % coordinates of the orthogonal projection 
%                          % of v onto span_C(z1,z2) with respect to the 
%                          % orthonormal basis {z1,z2tilda}
% w=v+[z1 z2tilda]*(-vc + U*vc);
% end


% V is a matrix; each column of V is tangent vector at z1
[nrV,ncV]=size(V);
if length(z1) ~= length(z2) || length(z1) ~= nrV
    error('vectors z1, z2 must have the same length as the number of rows in V');
end
s=z2'*z1;
z2=(s/abs(s))*z2;   % z2 is now z2star;
s=abs(s);   % inner porduct <z1,z2star>_C, which is positive real number,
            % so s=<z1,z2star>_C=<z2star,z1>_C.
b=sqrt(1-s*s);
z2tilda=(z2-s*z1)/b;  % z2tilda is a unit vector in span_C(z1,z2) such 
                      % that it is othogonal (over C) to z1, 
                      % i.e <z1,z2tilda>_C=0. 
U=[s -b; b s];
Vc=[z1'*V; z2tilda'*V];  % 2 x ncV matrix
W=V+[z1 z2tilda]*(-Vc + U*Vc);
end

